#ifndef LIBFAWK_EXECUTE_H
#define LIBFAWK_EXECUTE_H

#include <libfawk/libfawk.h>

/* Allocate (and return) a new cell on top of the stack, initialized to NIL */
FAWK_API fawk_cell_t *fawk_push_alloc(fawk_ctx_t *ctx);

/* Push a number or string on the stack, return the stack addess */
FAWK_API size_t fawk_push_num(fawk_ctx_t *ctx, fawk_num_t num);
FAWK_API size_t fawk_push_str(fawk_ctx_t *ctx, const char *str);

/* Remove the cell on top of the stack and copy it into dst. The caller
   needs to call fawk_cell_free() on dst. */
FAWK_API int fawk_pop(fawk_ctx_t *ctx, fawk_cell_t *dst);

/* Return a pointer to a cell in the stack, without changing the stack.
   If addr is negative, it is relative to the stack top, -1 pointing to
   the top item of the stack; if addr is 0 or positive, it is addressing
   the stack from the bottom. */
FAWK_API fawk_cell_t *fawk_peek(fawk_ctx_t *ctx, int addr);


/* Pop a number from the stack; if expect_num is zero, convert to num as needed
   (else assert) */
FAWK_API fawk_num_t fawk_pop_num(fawk_ctx_t *ctx, int expect_num);

/* Release resources used by the cell and set cell to NIL */
FAWK_API void fawk_cell_free(fawk_ctx_t *ctx, fawk_cell_t *cell);

/* Copy src to dst, increasing reference counters of strings and arrays as needed */
FAWK_API void fawk_cell_cpy(fawk_ctx_t *ctx, fawk_cell_t *dst, const fawk_cell_t *src);

/* Prepare a function call; returns 0 and modifies the stack on success.
   Error means funcname is not a function or does not exist. The caller
   needs to call fawk_call2() after a succesful call (else the stack
   items remain) */
FAWK_API int fawk_call1(fawk_ctx_t *ctx, const char *funcname);

/* Caller calls fawk_call1() then pushes args then calls fawk_call2() to
   set up VM states for executing the function. Returns 0 on succes.
   Error means argc was larger than the number of arguments the function
   takes (the stack is cleaned up in this case). The caller needs to call
   fawk_execute() if no error happened, that will execute the function and
   clean up the stack */
FAWK_API int fawk_call2(fawk_ctx_t *ctx, int argc);

/* Execute code as set up by fawk_call2(). Execute at most steps instructions
(-1 practically means no limit) */
FAWK_API fawk_execret_t fawk_execute(fawk_ctx_t *ctx, size_t steps);

/* Reset the VM so the stack and execution context is empty and a function
   can be called. This is the way to recover after execution limit is reached
   in fawk_execute(). */
FAWK_API void fawk_reset(fawk_ctx_t *ctx);

/* Eval to the argnth argument within a cfunc callback */
#define FAWK_CFUNC_ARG(argn) fawk_peek(ctx, -(argc-(argn)))

#endif /* LIBFAWK_EXECUTE_H */
